@@ -9,6 +9,12 @@ urlpatterns = patterns(
     url(r'^reports$',
         'reports',
         name='lava.scheduler.reports'),
+    url(r'^reports/failures$',
+        'failure_report',
+        name='lava.scheduler.failure_report'),
+    url(r'^reports/failures_json$',
+        'failed_jobs_json',
+        name='lava.scheduler.failed_jobs_json'),
     url(r'^active_jobs_json$',
         'index_active_jobs_json',
         name='lava.scheduler.active_jobs_json'),
@@ -81,6 +87,9 @@ urlpatterns = patterns(
     url(r'^job/(?P<pk>[0-9]+)/cancel$',
         'job_cancel',
         name='lava.scheduler.job.cancel'),
+    url(r'^job/(?P<pk>[0-9]+)/annotate_failure$',
+        'job_annotate_failure',
+        name='lava.scheduler.job.annotate_failure'),
     url(r'^job/(?P<pk>[0-9]+)/json$',
         'job_json',
         name='lava.scheduler.job.json'),
